/**
  **************************************************************************
  * @file     at32_video_ev_exint.c
  * @version  v2.0.2
  * @date     2022-09-15
  * @brief    at32_video_ev_exint program
  **************************************************************************
  *                       Copyright notice & Disclaimer
  *
  * The software Board Support Package (BSP) that is made available to
  * download from Artery official website is the copyrighted work of Artery.
  * Artery authorizes customers to use, copy, and distribute the BSP
  * software and its related documentation for the purpose of design and
  * development in conjunction with Artery microcontrollers. Use of the
  * software is governed by this copyright notice and the following disclaimer.
  *
  * THIS SOFTWARE IS PROVIDED ON "AS IS" BASIS WITHOUT WARRANTIES,
  * GUARANTEES OR REPRESENTATIONS OF ANY KIND. ARTERY EXPRESSLY DISCLAIMS,
  * TO THE FULLEST EXTENT PERMITTED BY LAW, ALL EXPRESS, IMPLIED OR
  * STATUTORY OR OTHER WARRANTIES, GUARANTEES OR REPRESENTATIONS,
  * INCLUDING BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY,
  * FITNESS FOR A PARTICULAR PURPOSE, OR NON-INFRINGEMENT.
  *
  **************************************************************************
  */

#include "at32_video_ev_exint.h"
#include "at32_video_ev_bf3901.h"

#define KEY_LED gpio_input_data_bit_read(GPIOA,GPIO_PINS_0)
#define LED_ON  gpio_bits_set(GPIOB, GPIO_PINS_9)
#define LED_OFF gpio_bits_reset(GPIOB, GPIO_PINS_9)
extern __IO u8  Line_Refresh_Flg;
extern volatile u16 RxBuffBankIndx ;
extern u8 DMA_Rx_Buffer[2][DMARxSize];
extern __IO u8 FrameCnt;
extern __IO u8 Frame_Refresh_Flg;
int abc_num = 0;
static u8 Key_flag = 0;
extern int Get_Frame;
extern int Key_run;

/**
  * @brief  this function handles external interrupt 15 to 10
  * @param  none
  * @retval none
  */
void EXINT15_10_IRQHandler(void)
{
#if BF3901_BOARD_VERSION == 1

  if (((EXINT->intsts & EXINT_LINE_14) != (uint32_t)RESET) )
  {
    delay_us(500);

    abc_num = SPI2->dt ;
    abc_num = SPI2->dt ;
    abc_num = SPI2->dt ;
    spi_i2s_flag_clear(SPI2, SPI_I2S_RDBF_FLAG);

    spi_enable(BF3901_SPI_SELECTED, FALSE);
    spi_enable(BF3901_SPI_SELECTED, TRUE);

    if(Get_Frame == 0)
    {
      BF3901_SPI_SLAVE_Rx_DMA_Channel->ctrl_bit.chen = FALSE;
      BF3901_SPI_SLAVE_Rx_DMA_Channel->maddr =  (uint32_t)(DMA_Rx_Buffer + RxBuffBankIndx);
      BF3901_SPI_SLAVE_Rx_DMA_Channel->dtcnt = DMARxSize;
      BF3901_SPI_SLAVE_Rx_DMA_Channel->ctrl_bit.chen = TRUE;

      RxBuffBankIndx ^= 1;
    }

    Frame_Refresh_Flg  = FRAME_START_SYNC_FLG;//FRAME_START_FLG;

  }

  EXINT->intsts = EXINT_LINE_14;
#elif BF3901_BOARD_VERSION == 2

  if (((EXINT->intsts & EXINT_LINE_12) != (uint32_t)RESET) )
  {
    delay_us(500);

    abc_num = SPI2->dt ;
    abc_num = SPI2->dt ;
    abc_num = SPI2->dt ;
    spi_i2s_flag_clear(SPI2, SPI_I2S_RDBF_FLAG);

    spi_enable(BF3901_SPI_SELECTED, FALSE);
    spi_enable(BF3901_SPI_SELECTED, TRUE);

    if(Get_Frame == 0)
    {
      BF3901_SPI_SLAVE_Rx_DMA_Channel->ctrl_bit.chen = FALSE;
      BF3901_SPI_SLAVE_Rx_DMA_Channel->maddr =  (uint32_t)(DMA_Rx_Buffer + RxBuffBankIndx);
      BF3901_SPI_SLAVE_Rx_DMA_Channel->dtcnt = DMARxSize;
      BF3901_SPI_SLAVE_Rx_DMA_Channel->ctrl_bit.chen = TRUE;

      RxBuffBankIndx ^= 1;
    }

    Frame_Refresh_Flg  = FRAME_START_SYNC_FLG;//FRAME_START_FLG;

  }

  EXINT->intsts = EXINT_LINE_12;
#endif
}

/**
  * @brief  this function handles external interrupt 0
  * @param  none
  * @retval none
  */
void EXINT0_IRQHandler(void)
{
  exint_flag_clear(EXINT_LINE_0);

  if(KEY_LED == 1)
  {
    EXINT->inten &= ~(1 << 0);

    if(Key_run != 0x66)
    {
      Key_run = 0x55;
    }
    else
    {
      Key_run = 0x11;
    }

    Line_Refresh_Flg = RX_IDLE_FLG;

    delay_us(10);

    if(KEY_LED == 1)
    {
      if(Key_flag != 1)
      {
        Key_flag = 1;
      }
      else
      {
        LED_OFF;
        Key_flag = 0;
      }
    }

    EXINT->inten |= (1 << 0);
  }
}

/**
  * @brief  initialize external interrupt
  * @param  none
  * @retval none
  */
void exintx_init(void)
{
  exint_init_type EXINT_InitStructure;

  crm_periph_clock_enable(CRM_IOMUX_PERIPH_CLOCK, TRUE);
  gpio_exint_line_config(GPIO_PORT_SOURCE_GPIOA, (GPIO_PINS_SOURCE0));

  EXINT_InitStructure.line_select = EXINT_LINE_0;
  EXINT_InitStructure.line_mode = EXINT_LINE_INTERRUPUT;
  EXINT_InitStructure.line_polarity = EXINT_TRIGGER_RISING_EDGE;
  EXINT_InitStructure.line_enable = TRUE;
  exint_init(&EXINT_InitStructure);

  nvic_irq_enable(EXINT0_IRQn, 0, 3);
}
