/**
  **************************************************************************
  * @file     at32_video_ev_lcd.h
  * @version  v2.0.2
  * @date     2022-09-15
  * @brief    at32_video_ev_lcd header file
  **************************************************************************
  *                       Copyright notice & Disclaimer
  *
  * The software Board Support Package (BSP) that is made available to
  * download from Artery official website is the copyrighted work of Artery.
  * Artery authorizes customers to use, copy, and distribute the BSP
  * software and its related documentation for the purpose of design and
  * development in conjunction with Artery microcontrollers. Use of the
  * software is governed by this copyright notice and the following disclaimer.
  *
  * THIS SOFTWARE IS PROVIDED ON "AS IS" BASIS WITHOUT WARRANTIES,
  * GUARANTEES OR REPRESENTATIONS OF ANY KIND. ARTERY EXPRESSLY DISCLAIMS,
  * TO THE FULLEST EXTENT PERMITTED BY LAW, ALL EXPRESS, IMPLIED OR
  * STATUTORY OR OTHER WARRANTIES, GUARANTEES OR REPRESENTATIONS,
  * INCLUDING BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY,
  * FITNESS FOR A PARTICULAR PURPOSE, OR NON-INFRINGEMENT.
  *
  **************************************************************************
  */

#ifndef __AT32_VIDEO_EV_LCD_H
#define __AT32_VIDEO_EV_LCD_H
#include "at32f403a_407_board.h"

//mcu=0,00: l8bit 0k
//mcu=1,01: 16bit	ok
//mcu=2,10: w4spi	ok
//mcu=3,11: w3spi	ok
//mcu=4,--: h8bit	ok
//mcu=5,--: reg16bit
#define  MCU 2

#define IMAGE_KAOLA_WIDTH            	   240
#define IMAGE_KAOLA_HEIGHT           	   240
#define IMAGE_PIXEL_TOTAL            		 (IMAGE_KAOLA_HEIGHT * IMAGE_KAOLA_WIDTH)

#define IMAGE_WIDTH_BUF        	         240*2
#define IMAGE_WIDTH            	         240
#define IMAGE_HEIGHT           	         240

#define LCD_WIDTH            	           240
#define LCD_HEIGHT           	           240

typedef struct
{
  u16 width;			//lcd width
  u16 height;			//lcd height
  u16 id;				  //lcd id
  u8  dir;			  //lcd direction
  u16	wramcmd;		//write ram cmd
  u16  setxcmd;		//set x coordinate cmd
  u16  setycmd;		//set y coordinate cmd
} _lcd_dev;

extern _lcd_dev lcddev;
/* point color and background color */
extern u16  POINT_COLOR;
extern u16  BACK_COLOR;

typedef struct
{
  u8 LCD_REG;
  u8 LCD_RAM;
} LCD_TypeDef;

#define LCD_BASE                         ((u32)(0x6C000000 | 0x000007FE))
#define LCD                              ((LCD_TypeDef *) LCD_BASE)

#define SPI_SCLK PAout(5)
#define SPI_SDA  PAout(7)

#define SPI_A0IN PAout(3)

#define	LCD_CS_SET  GPIOB->scr=1<<7      //CS_SET  PB7
#define	LCD_RS_SET	GPIOC->scr|=(1<<7)   //RS_SET  PC7	   
#define	LCD_SCL_SET	GPIOA->scr|=(1<<5)   //SCL_SET PA5
#define	LCD_SDA_SET	GPIOA->scr|=(1<<7)   //SDA_SET PA7
#define	LCD_RST_SET	GPIOA->scr|=(1<<15)  //RST_SET PA15

#define	LCD_CS_CLR  GPIOB->clr=1<<7      //CS_CLR  PB7
#define	LCD_RS_CLR	GPIOC->clr|=(1<<7)   //RS_CLR  PC7	   
#define	LCD_SCL_CLR	GPIOA->clr|=(1<<5)   //SCL_CLR PA5
#define	LCD_SDA_CLR	GPIOA->clr|=(1<<7)   //SDA_CLR PA7
#define	LCD_RST_CLR	GPIOA->clr|=(1<<15)  //RST_CLR PA15

#define 	DATAOUTD(x) 	GPIOD->odt=x;    //gpiod data out
#define 	DATAIND     		GPIOD->idt;    //gpiod data in

#define 	DATAOUTE(x) 	GPIOE->odt=x;    //gpioe data out
#define 	DATAINE     		GPIOE->idt;    //gpioe data in

/* scan direction, 0: 0, 1: 90, 2: 180, 3: 270 */
#define L2R_U2D  0 //left to right, up to down
#define L2R_D2U  1 //left to right, down to up
#define R2L_U2D  2 //right to left, up to down
#define R2L_D2U  3 //right to left, down to up

#define U2D_L2R  4 //up to down, left to right
#define U2D_R2L  5 //up to down, right to left
#define D2U_L2R  6 //down to up, left to right
#define D2U_R2L  7 //down to up, right to left

#if BF3901_BOARD_VERSION == 1
#define DFT_SCAN_DIR  L2R_U2D
#else
#define DFT_SCAN_DIR  R2L_D2U
#endif

/* color value */
#define WHITE         	                 0xFFFF
#define BLACK         	                 0x0000
#define BLUE         	                   0x001F
#define BRED                             0xF81F
#define GRED 			                       0xFFE0
#define GBLUE			                       0x07FF
#define RED           	                 0xF800
#define MAGENTA       	                 0xF81F
#define GREEN         	                 0x07E0
#define CYAN          	                 0x7FFF
#define YELLOW        	                 0xFFE0
#define BROWN 			                     0xBC40
#define BRRED 			                     0xFC07
#define GRAY  			                     0x8430

#define DARKBLUE      	                 0x01CF
#define LIGHTBLUE      	                 0x7D7C
#define GRAYBLUE       	                 0x5458

#define LIGHTGREEN     	                 0x841F
#define LIGHTGRAY                        0xEF5B
#define LGRAY 			                     0xC618

#define LGRAYBLUE                        0xA651
#define LBBLUE                           0x2B12

void lcd_ctrllinesconfig(void);
void lcd_reset(void);

/* 4 wire-spi */
void spi4w_writeregindex(u16 Index);
void spi4w_writedata(u16 dat);
void spi4w_writedisplaydata(u16 dat);

/* 8/16 bit */
void lcd_wr_reg(u16 regval);
void lcd_wr_8reg(u16 regval);
void lcd_wr_data(u16 data);
void lcd_wr_8data(u16 data);
void lcd_wr_reg_data(u16 data1, u16 data2);
void lcd_writereg(u16 LCD_Reg, u16 LCD_RegValue);

u8 lcd_rd_8data(void);

u16 lcd_rd_data(void);

u16 lcd_readreg(u16 lcd_reg);
void lcd_writeram_prepare(void);
void lcd_writeram(u16 rgb_code);
u16 lcd_bgr2rgb(u16 c);
void opt_delay(u8 i);
u16  lcd_readpoint(u16 x, u16 y);

void lcd_clear(u16 color);
void lcd_clear_part(u16 color);
void lcd_display(u16 color);
void lcd_frame(u16 rgb_data, u16 f_data);
void show_gray(void);

void check_crosstalk(void);
void check_flicker(void);

void lcd_fill(u16 sx, u16 sy, u16 ex, u16 ey, u16 color);
void lcd_color_fill(u16 sx, u16 sy, u16 ex, u16 ey, u16 *color);

void lcd_drawpoint(u16 x, u16 y);
void lcd_drawpoint_big(u16 x, u16 y);
void lcd_fast_drawpoint(u16 x, u16 y, u16 color);
void lcd_drawline(u16 x1, u16 y1, u16 x2, u16 y2);
void lcd_drawrectangle(u16 x1, u16 y1, u16 x2, u16 y2);
void lcd_draw_circle(u16 x0, u16 y0, u8 r);

void lcd_showchar(u16 x, u16 y, u8 num, u8 size, u8 mode);
u32 lcd_pow(u8 m, u8 n);
void lcd_shownum(u16 x, u16 y, u32 num, u8 len, u8 size);
void lcd_showxnum(u16 x, u16 y, u32 num, u8 len, u8 size, u8 mode);
void lcd_showstring(u16 x, u16 y, u16 width, u16 height, u8 size, u8 *p);

void lcd_setcursor(u16 xpos, u16 ypos);
void lcd_scan_dir(u8 dir);
void lcd_init(void);
void ili9341_ivo24_initial(void);
void blockwrite(unsigned int xstart, unsigned int xend, unsigned int ystart, unsigned int yend) ;
void lcd_set_window(u16 sx, u16 sy, u16 width, u16 height);

#endif

