/**
  **************************************************************************
  * @file     at32_video_ev_spi.c
  * @version  v2.0.2
  * @date     2022-09-15
  * @brief    at32_video_ev_spi program
  **************************************************************************
  *                       Copyright notice & Disclaimer
  *
  * The software Board Support Package (BSP) that is made available to
  * download from Artery official website is the copyrighted work of Artery.
  * Artery authorizes customers to use, copy, and distribute the BSP
  * software and its related documentation for the purpose of design and
  * development in conjunction with Artery microcontrollers. Use of the
  * software is governed by this copyright notice and the following disclaimer.
  *
  * THIS SOFTWARE IS PROVIDED ON "AS IS" BASIS WITHOUT WARRANTIES,
  * GUARANTEES OR REPRESENTATIONS OF ANY KIND. ARTERY EXPRESSLY DISCLAIMS,
  * TO THE FULLEST EXTENT PERMITTED BY LAW, ALL EXPRESS, IMPLIED OR
  * STATUTORY OR OTHER WARRANTIES, GUARANTEES OR REPRESENTATIONS,
  * INCLUDING BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY,
  * FITNESS FOR A PARTICULAR PURPOSE, OR NON-INFRINGEMENT.
  *
  **************************************************************************
  */

#include "at32_video_ev_spi.h"
#include "at32_video_ev_bf3901.h"

extern u16 Hv_sum;

u8  SPI_Rx_Tmp = 0;
volatile u16 RxBuffBankIndx = 0;
u8  DMA_Rx_Buffer[2][DMARxSize];
__IO u8  DMA_Refresh_Flg = 0;
u16  Data_Packet_Size = 0;
u16	LinePixelIdx = 0;
u8 DMA_tx_flag = 0;
extern __IO u16  RxIdx;

extern u8 FrameCnt;
extern __IO u8  Data_ID;
extern __IO u16 Image_Width;
extern __IO u16 Image_Height;
extern __IO u16 Line_Number;
extern __IO u16 Data_Packet_SizeBk;
extern __IO u8  Frame_Refresh_Flg;
extern __IO u8  Line_Refresh_Flg;
extern __IO u8  Fr_Status;
extern __IO u8  Ln_Status;

/**
  * @brief  dma1 channel4 IRQhandler
  * @param  none
  * @retval none
  */
void DMA1_Channel4_IRQHandler(void)
{
  if(DMA1->sts & BF3901_SPI_SLAVE_Rx_DMA_INT)
  {

    BF3901_SPI_SLAVE_Rx_DMA_Channel->ctrl_bit.chen = FALSE;
    Line_Refresh_Flg = LINE_START_FLG;
    dma_flag_clear(BF3901_SPI_SLAVE_Rx_DMA_INT);
  }
}

/**
  * @brief  dma1 channel3 IRQhandler
  * @param  none
  * @retval none
  */
void DMA1_Channel3_IRQHandler(void)
{
  if(dma_flag_get(LCD_SPI_MASTER_Tx_DMA_INT) == SET)
  {
    DMA_tx_flag = 1;
    dma_flag_clear(LCD_SPI_MASTER_Tx_DMA_INT);
  }
}

/**
  * @brief  initialize bf3901 spi
  * @param  none
  * @retval none
  */
void bf3901_spi_init(void)
{
  dma_init_type  DMA_InitStructure;
  gpio_init_type GPIO_InitStructure;
  spi_init_type  SPI_InitStructure;

  /* Enable SPI_SLAVE DMA clock */
  crm_periph_clock_enable(BF3901_SPI_SLAVE_DMA_CLK, TRUE);
  /* Enable GPIO clock for SPI_SLAVE */
  crm_periph_clock_enable( BF3901_SPI_GPIO_CLK, TRUE );
  crm_periph_clock_enable( CRM_IOMUX_PERIPH_CLOCK, TRUE );
  /* Enable SPI_SLAVE Periph clock */
  crm_periph_clock_enable( BF3901_SPI_CLK,	TRUE );

  gpio_default_para_init(&GPIO_InitStructure);
  GPIO_InitStructure.gpio_pins = BF3901_SPI_PIN_SCK;
  GPIO_InitStructure.gpio_mode = GPIO_MODE_INPUT;
  GPIO_InitStructure.gpio_pull = GPIO_PULL_DOWN;
  GPIO_InitStructure.gpio_drive_strength =  GPIO_DRIVE_STRENGTH_STRONGER;
  gpio_init(BF3901_SPI_GPIO, &GPIO_InitStructure);

  GPIO_InitStructure.gpio_pins = BF3901_SPI_PIN_MOSI;
  GPIO_InitStructure.gpio_mode = GPIO_MODE_MUX;
  GPIO_InitStructure.gpio_drive_strength =  GPIO_DRIVE_STRENGTH_STRONGER;
  gpio_init(BF3901_SPI_GPIO, &GPIO_InitStructure);

  /* bf3901_spi_slave_rx_dma_channel configuration */
  dma_reset(BF3901_SPI_SLAVE_Rx_DMA_Channel);
  DMA_InitStructure.buffer_size = VsyncSize;
  DMA_InitStructure.direction = DMA_DIR_PERIPHERAL_TO_MEMORY;
  DMA_InitStructure.memory_base_addr = (uint32_t)DMA_Rx_Buffer;
  DMA_InitStructure.memory_data_width = DMA_MEMORY_DATA_WIDTH_BYTE;
  DMA_InitStructure.memory_inc_enable = TRUE;
  DMA_InitStructure.peripheral_base_addr = (uint32_t)BF3901_SPI_SLAVE_DR_Base;
  DMA_InitStructure.peripheral_data_width = DMA_PERIPHERAL_DATA_WIDTH_BYTE;
  DMA_InitStructure.peripheral_inc_enable = FALSE;
  DMA_InitStructure.priority = DMA_PRIORITY_VERY_HIGH;
  DMA_InitStructure.loop_mode_enable = FALSE;
  dma_init(BF3901_SPI_SLAVE_Rx_DMA_Channel, &DMA_InitStructure);

  /* spi_slave configuration */
  spi_default_para_init(&SPI_InitStructure);
  SPI_InitStructure.transmission_mode = SPI_TRANSMIT_SIMPLEX_RX;
  SPI_InitStructure.master_slave_mode = SPI_MODE_SLAVE;
  SPI_InitStructure.mclk_freq_division = SPI_MCLK_DIV_2;
  SPI_InitStructure.first_bit_transmission = SPI_FIRST_BIT_MSB;
  SPI_InitStructure.frame_bit_num = SPI_FRAME_8BIT;
  SPI_InitStructure.clock_polarity = SPI_CLOCK_POLARITY_HIGH;
  SPI_InitStructure.clock_phase = SPI_CLOCK_PHASE_2EDGE;
  SPI_InitStructure.cs_mode_selection = SPI_CS_SOFTWARE_MODE;
  spi_init(BF3901_SPI_SELECTED, &SPI_InitStructure);
  spi_crc_polynomial_set(BF3901_SPI_SELECTED, 7);

  /* enable spi_slave rx request */
  spi_i2s_dma_receiver_enable(BF3901_SPI_SELECTED, TRUE);

  /* enable spi_slave */
  spi_enable(BF3901_SPI_SELECTED, TRUE);

  /* enable dma1 channel4 interrupt*/
  dma_interrupt_enable(BF3901_SPI_SLAVE_Rx_DMA_Channel, DMA_FDT_INT, TRUE);

}

#if(LCD_SPI_TX_DMA_EN)
u8  LcdSpiDMA_Tx_Buffer[LCDSPIDMATxSize];

/**
  * @brief  initialize lcd spi1
  * @param  none
  * @retval none
  */
void lcd_spi1_init(void)
{
  uint8_t i = 0;
  dma_init_type  DMA_InitStructure;
  gpio_init_type GPIO_InitStructure;
  spi_init_type  SPI_InitStructure;

  /* enable spi_master dma clock */
  crm_periph_clock_enable( LCD_SPI_MASTER_DMA_CLK, TRUE );
  crm_periph_clock_enable( LCD_SPI_SELECTED_CLK, TRUE );
  crm_periph_clock_enable( LCD_SPI_SELECTED_GPIO_CLK, TRUE );

  /* configure spi_master pins: sck, mosi and miso */
  /* configure miso pin as alternate function push pull */
  gpio_default_para_init(&GPIO_InitStructure);
  GPIO_InitStructure.gpio_pins = LCD_SPI_SELECTED_PIN_SCK | LCD_SPI_SELECTED_PIN_MOSI;
  GPIO_InitStructure.gpio_mode = GPIO_MODE_MUX;
  GPIO_InitStructure.gpio_drive_strength =  GPIO_DRIVE_STRENGTH_STRONGER;
  gpio_init(LCD_SPI_SELECTED_GPIO, &GPIO_InitStructure);

  /* configureand mosi pins as alternate function push pull */
  GPIO_InitStructure.gpio_pins = LCD_SPI_SELECTED_PIN_MISO;
  GPIO_InitStructure.gpio_mode = GPIO_MODE_INPUT;
  GPIO_InitStructure.gpio_pull = GPIO_PULL_DOWN;
  gpio_init(LCD_SPI_SELECTED_GPIO, &GPIO_InitStructure);

  gpio_bits_set(LCD_SPI_SELECTED_GPIO, LCD_SPI_SELECTED_PIN_MISO | LCD_SPI_SELECTED_PIN_SCK | LCD_SPI_SELECTED_PIN_MOSI);

  /* wait for clock pin pull down */
  for( i = 0; i < 200; i++);

  /* lcd_spi_master_tx_dma_channel configuration */
  dma_reset(LCD_SPI_MASTER_Tx_DMA_Channel);
  DMA_InitStructure.buffer_size = 38400;
  DMA_InitStructure.direction = DMA_DIR_MEMORY_TO_PERIPHERAL;
  DMA_InitStructure.memory_base_addr = (uint32_t)( (uint32_t)(DMA_Rx_Buffer) + HsyncSize);
  DMA_InitStructure.memory_data_width = DMA_MEMORY_DATA_WIDTH_BYTE;
  DMA_InitStructure.memory_inc_enable = TRUE;
  DMA_InitStructure.peripheral_base_addr = (uint32_t)LCD_SPI_MASTER_DR_Base;
  DMA_InitStructure.peripheral_data_width = DMA_PERIPHERAL_DATA_WIDTH_BYTE;
  DMA_InitStructure.peripheral_inc_enable = FALSE;
  DMA_InitStructure.priority = DMA_PRIORITY_HIGH;
  DMA_InitStructure.loop_mode_enable = FALSE;
  dma_init(LCD_SPI_MASTER_Tx_DMA_Channel, &DMA_InitStructure);

  /* spi_master configuration */
  spi_default_para_init(&SPI_InitStructure);
  SPI_InitStructure.transmission_mode = SPI_TRANSMIT_HALF_DUPLEX_TX;
  SPI_InitStructure.master_slave_mode = SPI_MODE_MASTER;
  SPI_InitStructure.mclk_freq_division = SPI_MCLK_DIV_2;
  SPI_InitStructure.first_bit_transmission = SPI_FIRST_BIT_MSB;
  SPI_InitStructure.frame_bit_num = SPI_FRAME_8BIT;
  SPI_InitStructure.clock_polarity = SPI_CLOCK_POLARITY_LOW;
  SPI_InitStructure.clock_phase = SPI_CLOCK_PHASE_1EDGE;
  SPI_InitStructure.cs_mode_selection = SPI_CS_SOFTWARE_MODE;
  spi_init(LCD_SPI_SELECTED, &SPI_InitStructure);
  spi_crc_polynomial_set(LCD_SPI_SELECTED, 7);

  /* enable spi_maser tx request */
  spi_i2s_dma_transmitter_enable(LCD_SPI_SELECTED, TRUE);
  /* enable spi_maser */
  spi_enable(LCD_SPI_SELECTED, TRUE);
  /* enable dma1 channel3 interrupt*/
  dma_interrupt_enable(LCD_SPI_MASTER_Tx_DMA_Channel, DMA_FDT_INT, TRUE);
}

/**
  * @brief  lcd spi1 send data
  * @param  txdata: send data
  * @retval none
  */
void lcd_spi1_senddata(u8 txdata)
{
  u16 retry = 0;

  retry = 0;

  /* check spi flag */
  while (spi_i2s_flag_get(LCD_SPI_SELECTED, SPI_I2S_TDBE_FLAG) == RESET)
  {
    retry++;

    if(retry > 2000)
    {
      return;
    }
  }

  LCD_RS_SET;
  LCD_SCL_CLR;
  LCD_SDA_SET;
  spi_i2s_data_transmit(LCD_SPI_SELECTED, txdata);
  retry = 0;

  while (spi_i2s_flag_get(LCD_SPI_SELECTED, SPI_I2S_BF_FLAG) == SET)
  {
    retry++;

    if(retry > 0xFFFE)
    {
      break;
    }
  }
}

/**
  * @brief  lcd spi1 send index
  * @param  txdata: index
  * @retval none
  */
void lcd_spi1_sendindex(u8 txdata)
{
  u16 retry = 0;

  retry = 0;

  while (spi_i2s_flag_get(LCD_SPI_SELECTED, SPI_I2S_TDBE_FLAG) == RESET)
  {
    retry++;

    if(retry > 2000)
    {
      return;
    }
  }

  LCD_RS_CLR;
  LCD_SCL_CLR;
  LCD_SDA_SET;
  spi_i2s_data_transmit(LCD_SPI_SELECTED, txdata);
  retry = 0;

  while (spi_i2s_flag_get(LCD_SPI_SELECTED, SPI_I2S_BF_FLAG) == SET)
  {
    retry++;

    if(retry > 0xFFFE)
    {
      break;
    }
  }
}
#else //(LCD_SPI_TX_DMA_EN)

/**
  * @brief  initialize lcd spi1
  * @param  none
  * @retval none
  */
void lcd_spi1_init(void)
{
  uint8_t i = 0;
  gpio_init_type GPIO_InitStructure;
  spi_init_type  SPI_InitStructure;

  crm_periph_clock_enable( LCD_SPI_SELECTED_CLK, TRUE );
  crm_periph_clock_enable( LCD_SPI_SELECTED_GPIO_CLK, TRUE );

  /* configure spi_master pins: sck, mosi and miso */
  /* configure miso pin as alternate function push pull */
  gpio_default_para_init(&GPIO_InitStructure);
  GPIO_InitStructure.gpio_pins = LCD_SPI_SELECTED_PIN_SCK | LCD_SPI_SELECTED_PIN_MOSI;
  GPIO_InitStructure.gpio_mode = GPIO_MODE_MUX;
  GPIO_InitStructure.gpio_drive_strength =  GPIO_DRIVE_STRENGTH_STRONGER;
  gpio_init(LCD_SPI_SELECTED_GPIO, &GPIO_InitStructure);

  /* configureand mosi pins as alternate function push pull */
  GPIO_InitStructure.gpio_pins = LCD_SPI_SELECTED_PIN_MISO;
  GPIO_InitStructure.gpio_mode = GPIO_MODE_INPUT;
  GPIO_InitStructure.gpio_pull = GPIO_PULL_DOWN;
  gpio_init(LCD_SPI_SELECTED_GPIO, &GPIO_InitStructure);

  gpio_bits_set(LCD_SPI_SELECTED_GPIO, LCD_SPI_SELECTED_PIN_MISO | LCD_SPI_SELECTED_PIN_SCK | LCD_SPI_SELECTED_PIN_MOSI);

  /* wait for clock pin pull down */
  for( i = 0; i < 200; i++);

  /* SPI_master configuration */
  spi_default_para_init(&SPI_InitStructure);
  SPI_InitStructure.transmission_mode = SPI_TRANSMIT_HALF_DUPLEX_TX;
  SPI_InitStructure.master_slave_mode = SPI_MODE_MASTER;
  SPI_InitStructure.mclk_freq_division = SPI_MCLK_DIV_4;
  SPI_InitStructure.first_bit_transmission = SPI_FIRST_BIT_MSB;
  SPI_InitStructure.frame_bit_num = SPI_FRAME_8BIT;
  SPI_InitStructure.clock_polarity = SPI_CLOCK_POLARITY_LOW;
  SPI_InitStructure.clock_phase = SPI_CLOCK_PHASE_1EDGE;
  SPI_InitStructure.cs_mode_selection = SPI_CS_SOFTWARE_MODE;
  spi_init(SPI1, &SPI_InitStructure);
  spi_crc_polynomial_set(SPI1, 7);

  spi_enable(LCD_SPI_SELECTED, TRUE);

}

/**
  * @brief  lcd spi1 send data
  * @param  txdata: send data
  * @retval none
  */
void lcd_spi1_senddata(u8 txdata)
{
  u16 retry = 0;

  retry = 0;

  while (spi_i2s_flag_get(LCD_SPI_SELECTED, SPI_I2S_TDBE_FLAG) == RESET)
  {
    retry++;

    if(retry > 2000)
    {
      return;
    }
  }

  LCD_RS_SET;
  LCD_SCL_CLR;
  LCD_SDA_SET;
  spi_i2s_data_transmit(LCD_SPI_SELECTED, TxData);
  retry = 0;

  while (spi_i2s_flag_get(LCD_SPI_SELECTED, SPI_I2S_BF_FLAG) == SET)
  {
    retry++;

    if(retry > 0xFFFE)
    {
      break;
    }
  }
}

/**
  * @brief  lcd spi1 send index
  * @param  txdata: index
  * @retval none
  */
void lcd_spi1_sendindex(u8 txdata)
{
  u16 retry = 0;
  retry = 0;

  while (spi_i2s_flag_get(LCD_SPI_SELECTED, SPI_I2S_TDBE_FLAG) == RESET)
  {
    retry++;

    if(retry > 2000)
    {
      return;
    }
  }

  LCD_RS_CLR;
  LCD_SCL_CLR;
  LCD_SDA_SET;
  spi_i2s_data_transmit(LCD_SPI_SELECTED, TxData);
  retry = 0;

  while (spi_i2s_flag_get(LCD_SPI_SELECTED, SPI_I2S_BF_FLAG) == SET)
  {
    retry++;

    if(retry > 0xFFFE)
    {
      break;
    }
  }
}
#endif

/**
  * @brief  lcd spi1 receive data
  * @param  none
  * @retval receive data
  */
u8 lcd_spi1_receivedata(void)
{
  u16 retry = 0;

  while (spi_i2s_flag_get(LCD_SPI_SELECTED, SPI_I2S_RDBF_FLAG) == RESET)
  {
    retry++;

    if(retry > 1000)
    {
      return 0;
    }
  }

  retry = spi_i2s_data_receive(LCD_SPI_SELECTED);
  return retry;
}
