/**
  **************************************************************************
  * @file     at32_video_ev_extend_sram.c
  * @version  v2.0.2
  * @date     2022-09-15
  * @brief    at32_video_ev_extend_sram program
  **************************************************************************
  *                       Copyright notice & Disclaimer
  *
  * The software Board Support Package (BSP) that is made available to
  * download from Artery official website is the copyrighted work of Artery.
  * Artery authorizes customers to use, copy, and distribute the BSP
  * software and its related documentation for the purpose of design and
  * development in conjunction with Artery microcontrollers. Use of the
  * software is governed by this copyright notice and the following disclaimer.
  *
  * THIS SOFTWARE IS PROVIDED ON "AS IS" BASIS WITHOUT WARRANTIES,
  * GUARANTEES OR REPRESENTATIONS OF ANY KIND. ARTERY EXPRESSLY DISCLAIMS,
  * TO THE FULLEST EXTENT PERMITTED BY LAW, ALL EXPRESS, IMPLIED OR
  * STATUTORY OR OTHER WARRANTIES, GUARANTEES OR REPRESENTATIONS,
  * INCLUDING BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY,
  * FITNESS FOR A PARTICULAR PURPOSE, OR NON-INFRINGEMENT.
  *
  **************************************************************************
  */

#include "at32_video_ev_extend_sram.h"
#include "at32f403a_407.h"
#include "stdio.h"

#define TEST_RAM_SIZE  0x800

/**
  * @brief  to extend sram size
  * @param  none
  * @retval none
  */
void extend_sram(void)
{
  /* target set_sram_96k is selected */
#ifdef EXTEND_SRAM_96K
  /* check if ram has been set to 96k, if not, change eopb0 */
  if(((USD->eopb0) & 0xFF) != 0xFF)
  {
    /* unlock option bytes program erase controller */
    flash_unlock();
    /* erase option bytes */
    flash_user_system_data_erase();
    /* change sram size to 96kb */
    flash_user_system_data_program((uint32_t)&USD->eopb0, 0xFF);
    NVIC_SystemReset();
  }

#endif

  /* target set_sram_224k is selected */
#ifdef EXTEND_SRAM_224K

  /* check if ram has been set to 224k, if not, change eopb0 */
  if(((USD->eopb0) & 0xFF) != 0xFE)
  {
    /* unlock option bytes program erase controller */
    flash_unlock();
    /* erase option bytes */
    flash_user_system_data_erase();
    /* change sram size to 224kb */
    flash_user_system_data_program((uint32_t)&USD->eopb0, 0xFE);
    NVIC_SystemReset();
  }

#endif
}

/**
  * @}
  */

/**
  * @}
  */

